% function [IntensityLimit outVol mask_lb mask_ub]=OutlierReduction(inVolName,bg,thresh,ratio,isverbose)
%
% Input inVolName is either a xml file containing dimension, type etc information of the 
% input volume or the 3D volume itself, or it is a 3D volume. bg= background value,
% outliers are replaced by bg. Thresh is a threshold, any data above thresh 
% is considered to be a potential outliers. Ratio is 1x2 vector of the 
% outlier ratio, e.g 1,2,5. Ratio(1) is the bottom % of data to be discarded,
% Ratio(2) is top % data(of histogram) to be discarded. bounds is a 2x1
% vector containing lower and upper bounds from inVolName. ISVERBOSE is a
% {0,1} flag indicating if the results will be shown. Default is off.

function [IntensityLimit outVol mask_lb mask_ub]=OutlierReduction(inVolName,bg,thresh,ratio,isverbose)

if nargin==3
    thresh=0;
elseif nargin==2
    thresh=0;
    bg=0;
elseif nargin==1
    fprintf('Not enough arguments. See "help OutlierReduction" \n');
    return;
elseif length(ratio)==1
    ratio=[ratio ratio];
end
try 
    x=isverbose;
catch e
    isverbose=0;
end
try
if ischar(inVolName)==1
    if (strcmp(inVolName(end-2:end),'xml'))
        [x px]=ReadXml(inVolName);
    elseif (strcmp(inVolName(end-2:end),'nii'))
        temp=load_nii(inVolName);
        x=double(temp.img);
        px.dim=size(x);
    else
        fprintf('Only xml and nii files are supported right now. ERROR. \n');
    end
else
    x=inVolName;
    px.dim=size(inVolName);    
end 
catch e
    fprintf('File can not be read \n');
    fprintf('%s\n',e.message);
end
    
N1=px.dim;
if length(N1)==2
    N1=[N1(1) N1(2) 1];
end
x=reshape(x,[1 N1(1)*N1(2)*N1(3)]);
mask_lb=zeros(size(x));
mask_ub=zeros(size(x));

T=length(find(x>thresh))*ratio/100;
N=length(unique(x));
x(x<=thresh)=0;
L=[min(x(:)) max(x(:))];


[H,P]=hist(x,N);
s=0;
count=2; % the thresh value is usually 0, it is in H(1)
if T(1)>0
    while s<T(1)
        s=s+H(count);
        count=count+1;
    end
    mask_lb(x<=P(count-1) & x>thresh)=1;
    x(x<=P(count-1))=bg;
    L(1)=P(count-1);
end

s=0;
count=N;
if T(2)>0
    while s<T(2)
        s=s+H(count);
        count=count-1;
    end
    mask_ub(x>=P(count+1) )=1;
    x(x>=P(count+1))=bg;
    L(2)=P(count+1);
end

% Outvolname=strcat(volname,'_nooutlier.raw');
% fp2=fopen(Outvolname,'w');
% fwrite(fp2,x,type);
% fclose(fp2);
% fclose(fp1);
if isverbose
if T(1)*T(2)~=0
    
    fprintf('Lower threshold=%g, Upper threshold=%g\n',L(1),L(2));
else
    if T(1)==0 && T(2)~=0
        L(1)=min(x);
        fprintf('Lower threshold=%g, Upper threshold=%g\n',min(x),L(2));
    elseif T(1)~=0 && T(2)==0
        L(2)=max(x);
        fprintf('Lower threshold=%g, Upper threshold=%g\n',L(1),max(x));
    else
        L(1)=min(x);L(2)=max(x);
        fprintf('Lower threshold=%g, Upper threshold=%g\n',min(x),max(x));
    end
        
end
end
outVol=reshape(x,[N1(1) N1(2) N1(3)]);
mask_ub=reshape(mask_ub,[N1(1) N1(2) N1(3)]);
mask_lb=reshape(mask_lb,[N1(1) N1(2) N1(3)]);
IntensityLimit=L;

clear temp;






    